#!/usr/bin/perl -w

# favnuts
# A quick hack to convert IE bookmarks to Netscape format
# Only tested on Netscape 4.7 under Linux!
# Although I have seen another program that did this it did
# not sort the bookmarks (a very important requirment for me).
# I decided to write this in Perl so that I could have them sorted

# Thanks to the author of FavToHtml John McCormick <johnmcc@gte.net>
# I used his code to work out the format to write Netscape's bookmark file in

# Release history
# Version: 0.1
# Date   : 12th December 1999
# First release

# Contact details
# email: darren@magma.dircon.co.uk
# url  : http://krapplets.cream.org/linux/

require strict;

&main;

sub main
{
    if ($#ARGV < 1)
    {
        print("Usage favnuts.pl: <IE bookmarks directory> <bookmarks.html>\n");
        exit(1);
    }
    else
    {
        my $path = AddSlash($ARGV[0]);
        my $bookmarks = $ARGV[1];
        my $indent = "";

        if (! -d $path)
        {
            die ("$path is not a directory\n");
        }

        open (BOOKMARKS, ">$bookmarks")
            || die "Unable to open $bookmarks for writing $!";
        &WriteHeader();
        FavesToHtml($path,$indent);
        &WriteFooter();

        close (BOOKMARKS);
    }
}

# Appends a slash to the filename if one doesn't exist
sub AddSlash($filename)
{
    my $filename = $_[0];

    if ($filename !~ /\/$/)
    {
        $filename .= "/";
    }
    return $filename;
}

# Reads the url string from the .url file
sub GetUrlInfo($urlfile)
{
    my $urlfile = $_[0];
    my $url = "";

    open (FILE, $urlfile) || return "";

    while (<FILE>)
    {
        if ($_ =~ /^URL=(\S*)/)
        {
            $url = $1;
            chomp($url);
        }
    }

    close (FILE);
    
    return $url;
}

# Called recursively to walk through the directories and write the bookmarks file
sub FavesToHtml($path,$indent)
{
    my %directories = ();
    my %urls = ();
    my $path = $_[0];
    my $entry = "";
    local *PATH;

    # Store the directories in one hash and the files in another
    # This is to sort the directories above the files
    opendir (PATH, $path) || die "Cannot open $path $!";
    while (defined($entry = readdir(PATH)))
    {
        my $fullpath = &AddSlash($path . $entry);
        if ( -d $fullpath)
        {
            if ($entry !~ /(\.|\.\.|Channels)/i)
            {
                $directories{$fullpath} = $entry;
            }
        }
    }
    closedir (PATH);

    opendir (PATH, $path) || die "Cannot open $path $!";
    while (defined($entry = readdir(PATH)))
    {
        my $fullpath = $path . $entry;
        if ( -f $fullpath)
        {
            if ($entry =~ /(.+)\.url$/)
            {
                my $desc = $1;
                $urls{$desc} = &GetUrlInfo($fullpath);

            }
        }
    }
    closedir (PATH);

    foreach $entry (sort { lc($a) cmp lc($b) } (keys %directories))
    {
        $indent .= "\t";
        # URLs in the Links directory appear on the toolbar in IE,
        # so put them in "Personal Toolbar Folder", Netscape's equivalent 
        if ($directories{$entry} =~ /^links$/i)
        {
            &WriteFolder("Personal Toolbar Folder",$indent);
        }
        else
        {
            &WriteFolder($directories{$entry},$indent);
        }
        # Recurse into subdirectory
        &FavesToHtml($entry,$indent);
    }

    # At the current level we have written all folder info, so now write URLs
    foreach $entry (sort { lc($a) cmp lc($b) } (keys %urls))
    {
        &WriteEntry($entry, $urls{$entry},$indent);
    }

    # indent just makes it look nice, Netscape will rewrite these, it's more there for debugging
    $indent =~ s/(.*)\t$/$1/;
    &WriteFooter($indent);
}

sub WriteFolder($title, $indent)
{
    my $title = $_[0];
    my $indent = $_[1];
    print BOOKMARKS "$indent<DT><H3 FOLDED>$title</H3>\n";
    print BOOKMARKS "$indent<DL><p>\n"
}

sub WriteEntry($description, $url, $indent)
{
    my $description = $_[0];
    my $url = $_[1];
    my $indent = $_[2];
    print BOOKMARKS "$indent<DT><A HREF=\"$url\">$description</A>\n";

}

sub WriteHeader()
{
    my $username = $ENV{"USER"};
    print BOOKMARKS "<TITLE>Bookmarks for $username</TITLE>\n";
    print BOOKMARKS "<H1>Bookmarks for $username</H1>\n";
    print BOOKMARKS "<DL><p>\n";
}

sub WriteFooter($indent)
{
    print BOOKMARKS "$indent</DL><p>\n"
}

